'回転強制振動解析V22の標準ﾓｼﾞｭｰﾙ        ［作成］黒田英夫(2023/03)
Imports System.Drawing.Printing

Module Rahm3VCom
    '対称回転体の強制振動解析
    '偏心および慣性主軸ずれの不釣合い力による強制振動
    'たわみ振動と傾き振動が連成
    '剪断変形、回転慣性、ジャイロモーメントを考慮
    'ＭＣＫ型の振動方程式
    'Ｍ(d^2ｕ／dt^2)＋Ｃ(dｕ／dt)＋Ｋｕ＝Ｆ
    'Ｃ行列にはジャイロマトリックスと減衰係数を含む

    'V11での改良点・・・変位図と応力図でマウス移動（Shiftキー併用）の説明を追加、
    '                   変位図で変位作画の倍率を変更、かつ構造図有無の選択を追加、
    '                   応力図は変位後の図上の作画に変更
    'V12での改良点・・・各画面でボタンを読みやすい色に改善
    'V13での改良点・・・３次元グラフィック図で回転操作性の改善、
    '                   アニメーション画面で回転操作後に作画されないバグを修正
    'V14での改良点・・・Visual Basic 2017に変更
    'V15での改良点・・・解析計算開始前に入力データのエラーチェックを追加
    'V16での改良点・・・X3Dファイル出力の文字コードをShift-JISからUTF8に変更
    'V17での改良点・・・プログラムファイルの文字コードをUTF-8に統一、
    '                   データファイルの文字コードを標準でUTF-8に変更
    '                   （従来のShift-JISもオプションで可能）
    'V18での改良点・・・データ表画面で表計算ソフトから貼り付け時に次行に0が入るのを防止
    'V19での改良点・・・入力データのファイル印刷メニューを廃止
    '                   （文字コードによる不具合のため）
    'V20での改良点・・・グラフ用データの作成を追加
    'V21での改良点・・・中空軸を追加、
    '                   UTF-8の入力データファイルをX3Dファイルへ変換できないバグを修正
    'V22での改良点・・・フリーソフト化、VB2022に移行、文字コードはUTF-8のみ採用

    Public Const Dpai As Single = 6.283185 '2π
    Public Const Pai As Single = 3.141593  'π

    Public CalcRes$         '計算結果表示データ
    Public Cr$                      'CR+LFのコード
    Public DirOpen$         '立ち上げ時のディレクトリー名
    Public Fgraph$                  'ｸﾞﾗﾌﾃﾞｰﾀの出力ﾌｧｲﾙ名
    Public Fm$              '入力データのファイル名
    Public Fy$              '計算結果のファイル名
    Public Title$           'タイトル

    Public Timst As Date          '計算開始時刻

    Public InOut As Integer         'ファイル入出力の種類（0:入力、1:出力、2:RotatVibﾌｧｲﾙ入力）
    Public KdataBackup As Integer   '入力データのバックアップファイル作成（0:なし、1:有り）
    Public KdataChange As Integer   'データ変更の有無（0:なし、1:有り）
    Public KE1 As Integer           '開始要素番号
    Public KE2 As Integer           '最終要素番号
    Public KfileOK As Integer       'ﾌｧｲﾙのOK有無（0:なし、1:有り）
    '    Public KindEncod As Integer     'ファイルの文字コード（0:UTF-8、1:Shif-JIS）
    Public KindGraph As Integer     'ｸﾞﾗﾌ種類（1:１節点多変数、2:多節点１変数）
    Public KindPost As Integer      '後処理の種類（1:変位図、2:応力図、3:ｱﾆﾒｰｼｮﾝ、
    '                                4:主要値表示、5:ｸﾞﾗﾌﾃﾞｰﾀの作成）
    Public KmouseDchange As Integer 'ﾏｳｽ指定要素のﾃﾞｰﾀ変更有無
    Public KmouseDset As Integer    'ﾏｳｽ指定要素のﾃﾞｰﾀ項目（1:材料番号、2:拘束節点、3:バネ支持節点、
    '                                4:減衰節点）
    Public KmouseElem As Integer    'ﾏｳｽ指定の要素番号
    Public KmousePoint As Integer   'ﾏｳｽ指定の節点番号
    Public KnewUFileA As Integer    '最近使用ﾌｧｲﾙAの更新有無
    Public KnumDisp As Integer      '構造作画での番号表示（0:なし、1:有り）
    Public KnumSb As Integer        'ﾌｫﾝﾄ文字ｻｲｽﾞの操作変数
    Public KnumSize As Integer      '構造作画での番号ｻｲｽﾞ
    Public KmeshDisp As Integer     '後処理する回転数番号
    Public Kscreen As Integer       '作画画面サイズ（0：全画面、1：通常画面）
    Public Kvalue As Integer        '作画の変数ｲﾝﾃﾞｯｸｽ
    Public KvalueG As Integer       'ｸﾞﾗﾌﾃﾞｰﾀの変数ｲﾝﾃﾞｯｸｽ
    Public MeshRev As Integer       '回転数の分割数
    Public Mprop As Integer         '材料データの成分数(=4)
    Public Nall As Integer          '変数の総数
    Public Nbound, Nboundb As Integer        '拘束節点数
    Public Ndamp, Ndampb As Integer         '減衰節点数
    Public Ndisc As Integer         '剛性円板数
    Public Ndim As Integer          '１節点当りの変数の数(=4)
    Public Nelem As Integer         '要素数
    Public NKmax As Integer         '最大繰り返し計算回数
    Public Nmat As Integer          '材料データ数
    Public Norder As Integer        '正方行列の次数
    Public NPgraph As Integer       'ｸﾞﾗﾌ節点指定（節点番号または節点数）
    Public Npoint As Integer        '節点数（=Nelem+1）
    Public Nspring, Nspringb As Integer       'バネ支持節点数
    Public NumLmax As Integer       '小行列の最大次数
    Public Nvar As Integer          '１要素当りの変数の総数
    Public Nwheel As Integer        '剛性円輪数

    Public Cfix As Single           'XY方向ﾋﾟｸｾﾙ差の補正係数
    Public DistFactor As Single     '視点とモデルとの距離倍率（標準1）
    Public DispRatio As Single      '変位作画の倍率
    Public DOmegat As Single        '角度刻み（°）
    Public EpsCalc As Single        '収束基準誤差率
    Public EyeAngle As Single       '視点と原点を結ぶ線の水平面との角度
    Public Eyeh As Single           '視点高さ
    Public HFactor As Single        '視点の高さ倍率（基準1）
    Public RevDisp As Single        '結果図作画の回転数
    Public RevMax As Single         '回転数MAX
    Public RevMin As Single         '回転数MIN
    Public Rlenmax As Single        'ﾓﾃﾞﾙの1軸の最大長さ
    Public Rlenmodel As Single      '視点の距離（座標系間）
    Public XaxisMax As Single       'Ｘ軸の最大値
    Public XaxisMin As Single       'Ｘ軸の最小値
    Public Xlower As Single         '作画領域のX最小値
    Public XRotate As Single        'モデルのX軸回りの回転角（右ねじ正、°）
    Public Xupper As Single         '作画領域のX最大値
    Public XYscale As Single        '作画スケール
    Public YaxisMax As Single       'Ｙ軸の最大値
    Public YaxisMin As Single       'Ｙ軸の最小値
    Public Ylower As Single         '作画領域のY最小値
    Public YRotate As Single        'モデルのY軸回りの回転角（右ねじ正、°）
    Public Yupper As Single         '作画領域のY最大値
    Public ZRotate As Single        'モデルのZ軸回りの回転角（右ねじ正、°）

    Public UFileA(6) As String      '最近使用したﾃﾞｰﾀﾌｧｲﾙの保存配列

    Public Jdirect(,) As Integer    'Y,Z方向の拘束有無（0:なし、1:有り）
    Public KdispMatR() As Integer   '固有値計算行列の変数番号対応の変位方向
    Public KdrawElem() As Integer   '各要素の作画有無（0:なし、1:有り）
    Public KPdamp() As Integer      '減衰の節点番号
    Public KPdisc() As Integer      '剛性円板取付の節点番号
    Public KPgraph(20) As Integer   'ｸﾞﾗﾌ節点番号
    Public KPspring() As Integer    'バネ支持の節点番号
    Public KPwheel() As Integer     '剛性円輪取付の節点番号
    Public Nodemb() As Integer      '全体行列の変数番号対応の節点番号
    Public NodeMatR() As Integer    '固有値計算行列の変数番号対応の節点順位
    Public NPrefer() As Integer     '要素計算における節点変数の対応番号
    Public Nrest() As Integer       '拘束の節点番号
    Public NumMat() As Integer      '各要素の材料種類番号

    Public Alpy() As Single         'Y方向変位の初期角
    Public Alpz() As Single         'Z方向変位の初期角
    Public AxisMax(3) As Single     'モデルのXYZ各軸の最大値
    Public AxisMin(3) As Single     'モデルのXYZ各軸の最小値
    Public Cdamp(,) As Single       'Y,Z方向の減衰係数
    Public CenterPos(3) As Single   '構造中心の座標
    Public Cspring(,) As Single     'Y,Z方向のバネ定数
    Public DataElem(,) As Single    '各要素の不釣合い量
    Public DataMat(,) As Single     '各材料の物性値
    Public DiscData(,) As Single    '剛性円板のデータ
    Public RPy() As Single          'Y方向変位の振幅
    Public RPz() As Single          'Z方向変位の振幅
    Public Tdisp(,) As Single       '結果の変位
    Public TransMat(4, 4) As Single 'モデルの合成アフィン変換行列
    Public Tu() As Single           '結果の変数値
    Public WheelData(,) As Single   '剛性円輪のデータ
    Public Xdata() As Single        '節点X座標
    Public Xmodel(), Ymodel(), Zmodel() As Single 'モデル座標における節点座標
    Public Xmouse(2), Ymouse(2) As Single   'ﾏｳｽ位置座標
    Public Xnew(), Ynew(), Znew() As Single   '変位入りの節点座標
    Public Xscreen(), Yscreen() As Single   'スクリーンにおける節点座標
    Public Xview(), Yview(), Zview() As Single  '視点座標における節点座標（左手系）

    Sub AdjustRatio(ByRef ratio As Single)
        '変位比率の調整
        Dim nb As Integer
        Dim b1, b2, std As Single

        std = 0.05
        b1 = std / ratio
        If b1 < 1.0 Then
            ratio = 1.0
        ElseIf b1 < 10.0 Then
            b2 = b1 * 10
            ratio = Int(b2) / 10.0
        Else    '上位2桁を採用
            b2 = b1
            nb = 0
            Do
                b2 = b2 / 10
                nb = nb + 1
                If b2 < 10 Then
                    Exit Do
                End If
            Loop
            b2 = Int(b2 * 10)
            If nb < 2 Then
                ratio = b2
            Else
                ratio = b2 * 10 ^ (nb - 1)
            End If
        End If

    End Sub

    Sub AxisCheck(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef rlenmaxb As Single, ByRef xd() As Single, _
    ByRef yd() As Single, ByRef zd() As Single)
        'X、Y、Z軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'rlenmaxb･･･1軸の最大長さ
        'xd(),yd(),zd()･･･節点の座標
        Dim ib, ie, ii, ik, j As Integer
        Dim b1 As Single
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    ib = ie + j - 1
                    If ik = 0 Then
                        ik = 1
                        AxisMin(1) = xd(ib)
                        AxisMin(2) = yd(ib)
                        AxisMin(3) = zd(ib)
                        For ii = 1 To 3
                            AxisMax(ii) = AxisMin(ii)
                        Next ii
                    Else
                        For ii = 1 To 3
                            Select Case ii
                                Case 1
                                    b1 = xd(ib)
                                Case 2
                                    b1 = yd(ib)
                                Case 3
                                    b1 = zd(ib)
                            End Select
                            If b1 < AxisMin(ii) Then
                                AxisMin(ii) = b1
                            ElseIf b1 > AxisMax(ii) Then
                                AxisMax(ii) = b1
                            End If
                        Next ii
                    End If
                Next j
            End If
        Next ie

        rlenmaxb = AxisMax(1) - AxisMin(1)
        For ii = 2 To 3
            b1 = AxisMax(ii) - AxisMin(ii)
            If b1 > rlenmaxb Then
                rlenmaxb = b1
            End If
        Next ii

    End Sub

    Sub AxisCheckS(ByVal k1 As Integer, ByRef k2 As Integer, ByRef xmin As Single, _
    ByRef xmax As Single, ByRef ymin As Single, ByRef ymax As Single, _
    ByRef xdb() As Single, ByRef ydb() As Single)
        'X･Y座標の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        'xdb(),ydb()･･･節点座標
        Dim i, ie, ik, j As Integer
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    i = ie + j - 1
                    If ik = 0 Then
                        ik = 1
                        xmin = xdb(i) : xmax = xmin
                        ymin = ydb(i) : ymax = ymin
                    Else
                        If xdb(i) < xmin Then
                            xmin = xdb(i)
                        ElseIf xdb(i) > xmax Then
                            xmax = xdb(i)
                        End If
                        If ydb(i) < ymin Then
                            ymin = ydb(i)
                        ElseIf ydb(i) > ymax Then
                            ymax = ydb(i)
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub AxisCheckX(ByVal k1 As Integer, ByVal k2 As Integer,
    ByRef rlenmaxb As Single, ByRef xd() As Single)
        'X軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'rlenmaxb･･･1軸の最大長さ
        'xd()･･･節点の座標
        Dim ib, ie, ik, j As Integer
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    ib = ie + j - 1
                    If ik = 0 Then
                        ik = 1
                        AxisMin(1) = xd(ib)
                        AxisMax(1) = AxisMin(1)
                    Else
                        If xd(ib) < AxisMin(1) Then
                            AxisMin(1) = xd(ib)
                        ElseIf xd(ib) > AxisMax(1) Then
                            AxisMax(1) = xd(ib)
                        End If
                    End If
                Next j
            End If
        Next ie

        rlenmaxb = AxisMax(1) - AxisMin(1)

    End Sub

    Sub AxisSet(ByVal dispc As Single, ByRef tdb(,) As Single)
        '座標の設定
        'dispc･･･係数
        'tdb(,)･･･Y･Z方向の変位
        Dim i As Integer
        For i = 1 To Npoint
            Xnew(i) = Xdata(i)
            Ynew(i) = dispc * tdb(i, 1)
            Znew(i) = dispc * tdb(i, 2)
        Next i
    End Sub

    Sub CfileDispU(ByVal fxx$)
        '計算結果ﾌｧｲﾙから変位ﾃﾞｰﾀの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･計算結果ﾌｧｲﾙ名
        Dim aa$
        Dim ib, ip As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        Do
            Do
                aa$ = sr.ReadLine
                dat = aa.Split(",")
            Loop Until dat(0) = "回転数のメッシュ番号"
            ib = dat(1)
            If ib = KmeshDisp Then
                Exit Do
            End If
        Loop
        Do
            aa$ = sr.ReadLine
        Loop Until aa$ = "＊＊ 最大変位時の出力 ＊＊"
        aa$ = sr.ReadLine
        aa$ = sr.ReadLine
        aa$ = sr.ReadLine
        aa$ = sr.ReadLine
        For ip = 1 To Npoint
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Tdisp(ip, 1) = dat(1)
            Tdisp(ip, 2) = dat(2)
        Next ip

        sr.Close()

    End Sub

    Sub ColorSet(ByVal kb As Integer, ByRef colorx As Color)
        '色の設定
        'kb･･･指定番号（1～）
        'colorx･･･色
        Dim kcount As Integer
        kcount = (kb - 1) Mod 24
        Select Case kcount
            Case 0
                colorx = Color.Black
            Case 1
                colorx = Color.Green
            Case 2
                colorx = Color.DarkOrange
            Case 3
                colorx = Color.FromArgb(0, 0, 128)
            Case 4
                colorx = Color.FromArgb(128, 0, 0)
            Case 5
                colorx = Color.FromArgb(192, 99, 0)
            Case 6
                colorx = Color.Violet
            Case 7
                colorx = Color.DarkBlue
            Case 8
                colorx = Color.FromArgb(90, 90, 90)
            Case 9
                colorx = Color.DarkGreen
            Case 10
                colorx = Color.DarkRed
            Case 11
                colorx = Color.Brown
            Case 12
                colorx = Color.FromArgb(0, 128, 99)
            Case 13
                colorx = Color.FromArgb(192, 99, 99)
            Case 14
                colorx = Color.LimeGreen
            Case 15
                colorx = Color.Purple
            Case 16
                colorx = Color.Gray
            Case 17
                colorx = Color.FromArgb(99, 99, 192)
            Case 18
                colorx = Color.FromArgb(160, 128, 0)
            Case 19
                colorx = Color.FromArgb(160, 128, 99)
            Case 20
                colorx = Color.BurlyWood
            Case 21
                colorx = Color.DarkSeaGreen
            Case 22
                colorx = Color.DarkKhaki
            Case 23
                colorx = Color.DarkOrchid
        End Select

    End Sub

    Sub CoordiChange(ByVal rlenb As Single)
        '節点座標の視点座標への変換
        'rlenb･･･ﾓﾃﾞﾙ座標原点と視点座標との距離
        Dim ip As Integer
        For ip = 1 To Npoint
            Xview(ip) = Xmodel(ip)
            Yview(ip) = Zmodel(ip)
            Zview(ip) = Ymodel(ip) + rlenb
        Next ip

    End Sub

    Sub DataCheck(ByVal kdisp As Integer, ByRef num As Integer)
        '入力ﾃﾞｰﾀのｴﾗｰﾁｪｯｸ
        'kdisp･･･エラー無しのメッセージの有無（0:なし、1:有り）
        'num･･･エラーの数（上限10）
        Dim a2$, aa$, aerr1$
        Dim i, ib, icc, ie, ip, j, kforce, nb As Integer
        Dim b1 As Single
        Dim abuf$(10)

        a2$ = ""
        aa$ = ""
        aerr1$ = "になっています！"
        num = 0
        kforce = 0

        If Nelem < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  要素数が" + Str$(Nelem) + aerr1$
        End If
        If Nbound < 1 And Nspring = 0 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  拘束節点数が" + Str$(Nbound) + aerr1$
        End If
        If Nmat < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  材料データ数が" + Str$(Nmat) + aerr1$
        End If

        '要素のﾁｪｯｸ
        For ie = 1 To Nelem
            nb = NumMat(ie)
            If nb < 1 Or nb > Nmat Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                aa$ = aa$ + "  要素番号" + Str$(ie) + "で"
                abuf$(num) = aa$ + " 材料番号が" + Str$(nb) + aerr1$
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
            If DataElem(ie, 1) <> 0 Or DataElem(ie, 3) <> 0 Then
                kforce = 1
            End If
        Next ie

        '節点のﾁｪｯｸ
        For ip = 1 To Nelem
            For j = ip + 1 To Nelem + 1
                b1 = Xdata(ip) - Xdata(j)
                If b1 >= 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  節点番号" + Str$(ip) + "、" + Str$(j) + "の"
                    abuf$(num) = aa$ + " 座標関係が間違っています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        Next ip

        If Ndisc > 0 Then
            '剛性円板ﾃﾞｰﾀのﾁｪｯｸ
            For i = 1 To Ndisc
                nb = KPdisc(i)
                If nb < 1 Or nb > Nelem + 1 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  剛性円板の番号" + Str$(i) + "で"
                    abuf$(num) = aa$ + " 取付節点番号が" + Str$(nb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                For j = 1 To 4
                    b1 = DiscData(i, j)
                    If b1 <= 0 Then
                        Select Case j
                            Case 1
                                a2$ = " 密度が"
                            Case 2
                                a2$ = " 外径が"
                            Case 3
                                a2$ = " 内径が"
                            Case 4
                                a2$ = " 厚さが"
                        End Select
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  剛性円板の番号" + Str$(i) + "で" + a2$
                        abuf$(num) = aa$ + Str$(b1) + aerr1$
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                Next j
                If DiscData(i, 5) <> 0 Or DiscData(i, 7) <> 0 Then
                    kforce = 1
                End If
            Next i
        End If

        If Nwheel > 0 Then
            '剛性円輪ﾃﾞｰﾀのﾁｪｯｸ
            For i = 1 To Nwheel
                nb = KPwheel(i)
                If nb < 1 Or nb > Nelem + 1 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  剛性円輪の番号" + Str$(i) + "で"
                    abuf$(num) = aa$ + " 取付節点番号が" + Str$(nb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                b1 = WheelData(i, 1)
                If b1 <= 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  剛性円輪の番号" + Str$(i) + "で"
                    abuf$(num) = aa$ + "密度が" + Str$(b1) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                For j = 2 To 13
                    b1 = WheelData(i, j)
                    If b1 < 0 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  剛性円輪の番号" + Str$(i) + "で"
                        abuf$(num) = aa$ + "負の寸法があります！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                Next j
                b1 = (WheelData(i, 6) + WheelData(i, 7)) * 0.9999
                If b1 > WheelData(i, 3) Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  剛性円輪の番号" + Str$(i) + "で"
                    abuf$(num) = aa$ + "厚さが間違っています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                If WheelData(i, 14) <> 0 Or WheelData(i, 16) <> 0 Then
                    kforce = 1
                End If
            Next i
        End If

        '材料ﾃﾞｰﾀのﾁｪｯｸ
        For i = 1 To Nmat
            For j = 1 To Mprop
                b1 = DataMat(i, j)
                If b1 <= 0 And (j <> 2 And j <> 5) Then
                    Select Case j
                        Case 1
                            a2$ = " ヤング率が"
                        Case 3
                            a2$ = " 密度が"
                        Case 4
                            a2$ = " 直径が"
                    End Select
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  材料番号" + Str$(i) + "で" + a2$
                    abuf$(num) = aa$ + Str$(b1) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        Next i

        If Nbound > 0 Then
            '拘束ﾃﾞｰﾀのﾁｪｯｸ
            For ib = 1 To Nbound
                nb = Nrest(ib)
                If nb < 1 Or nb > Nelem + 1 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  拘束番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 節点番号が" + Str$(nb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                For j = ib + 1 To Nbound
                    If nb = Nrest(j) Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  拘束番号" + Str$(ib) + "、" + Str$(j) + "で"
                        abuf$(num) = aa$ + " 節点番号" + Str$(nb) + "が重複しています！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                Next j
                icc = 0
                For j = 1 To 2
                    If Jdirect(ib, j) > 0 Then
                        icc = 1
                        Exit For
                    End If
                Next j
                If icc = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  拘束番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 拘束がありません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If Nspring > 0 Then   'ﾊﾞﾈ支持節点のﾁｪｯｸ
            For ib = 1 To Nspring
                nb = KPspring(ib)
                If nb < 1 Or nb > Nelem + 1 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  バネ支持の番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 節点番号が" + Str$(nb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                icc = 0
                For j = 1 To 2
                    b1 = Cspring(ib, j)
                    If b1 > 0 Then
                        icc = 1
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  バネ支持の番号" + Str$(ib) + "で"
                        aa$ = aa$ + " バネ定数が正" + aerr1$
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    ElseIf b1 < 0 Then
                        icc = 1
                    End If
                Next j
                If icc = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  バネ支持の番号" + Str$(ib) + "で"
                    aa$ = aa$ + " バネ定数が設定されていません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If Ndamp > 0 Then '減衰節点のﾁｪｯｸ
            For ib = 1 To Ndamp
                nb = KPdamp(ib)
                If nb < 1 Or nb > Nelem + 1 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  減衰の番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 節点番号が" + Str$(nb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                icc = 0
                For j = 1 To 2
                    b1 = Cdamp(ib, j)
                    If b1 < 0 Then
                        icc = 1
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  減衰の番号" + Str$(ib) + "で"
                        abuf$(num) = aa$ + " 減衰係数が負" + aerr1$
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    ElseIf b1 > 0 Then
                        icc = 1
                    End If
                    If icc = 0 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  減衰の番号" + Str$(ib) + "で"
                        abuf$(num) = aa$ + " 減衰係数が設定されていません！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                Next j
            Next ib
        End If

        If kforce = 0 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  不釣合いが設定されていません！"
        End If

        If num = 0 Then
            If kdisp = 1 Then
                aa$ = "データのエラーチェックが終了しました！"
                MsgBox(aa$, 0, "データＯＫ")
            End If
        Else
            Call ErrorDisp(0, num, abuf$) 'ｴﾗｰの表示
        End If

    End Sub

    Sub DataStoreCheck(ByRef ix As Integer)
        'ﾃﾞｰﾀ保存の確認
        'ix･･･返答（6:yes、それ以外は中止）
        ix = 0
        If KdataChange > 0 Then
            Dim aa$
            aa$ = Fm$ + "ファイルが変更されていますが、保存なしでよいですか？"
            ix = MsgBox(aa$, 4, "データ変更の警告")
            If ix = 6 Then
                KdataChange = 0
            End If
        End If

    End Sub

    Sub DfileInputU(ByVal fxx$)
        '構造ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        Dim aa$
        Dim i, ib, ie, ip, j, mpb As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) <> "プログラム名" Then
            aa$ = "本ソフトのUTF-8データファイルとは違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        If dat(1) <> "RotatForceVib" Then
            aa$ = "データファイルが違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        Title$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nelem = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ndisc = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nwheel = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nmat = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nbound = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nspring = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ndamp = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ndim = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        'Mprop = dat(1)
        mpb = dat(1)
        Mprop = 5

        ReDim NumMat(Nelem), DataElem(Nelem, 4), Xdata(Nelem + 1)
        ReDim KPdisc(Ndisc), DiscData(Ndisc, 8)
        ReDim KPwheel(Nwheel), WheelData(Nwheel, 17)
        ReDim DataMat(Nmat, Mprop)
        ReDim Nrest(Nbound), Jdirect(Nbound, 2)
        ReDim KPspring(Nspring), Cspring(Nspring, 2)
        ReDim KPdamp(Ndamp), Cdamp(Ndamp, 2)

        aa$ = sr.ReadLine
        For ie = 1 To Nelem
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NumMat(ie) = dat(1)
            For j = 1 To 4
                DataElem(ie, j) = dat(j + 3)
            Next j
        Next ie

        aa$ = sr.ReadLine
        For ip = 1 To Nelem + 1
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Xdata(ip) = dat(1)
        Next ip

        If Ndisc > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Ndisc
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KPdisc(i) = dat(1)
                For j = 1 To 8
                    DiscData(i, j) = dat(j + 1)
                Next j
            Next i
        End If

        If Nwheel > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Nwheel
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KPwheel(i) = dat(3)
                aa$ = sr.ReadLine
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                For j = 1 To 7
                    WheelData(i, j) = dat(j - 1)
                Next j
                aa$ = sr.ReadLine
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                For j = 8 To 17
                    WheelData(i, j) = dat(j - 8)
                Next j
            Next i
        End If

        aa$ = sr.ReadLine
        For i = 1 To Nmat
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            For j = 1 To mpb
                DataMat(i, j) = dat(j)
            Next j
        Next i

        If Nbound > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Nbound
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Nrest(i) = dat(1)
                Jdirect(i, 1) = dat(2)
                Jdirect(i, 2) = dat(3)
            Next i
        End If

        If Nspring > 0 Then
            aa$ = sr.ReadLine
            For ib = 1 To Nspring
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KPspring(ib) = dat(1)
                Cspring(ib, 1) = dat(2)
                Cspring(ib, 2) = dat(3)
            Next ib
        End If

        If Ndamp > 0 Then
            aa$ = sr.ReadLine
            For ib = 1 To Ndamp
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KPdamp(ib) = dat(1)
                Cdamp(ib, 1) = dat(2)
                Cdamp(ib, 2) = dat(3)
            Next ib
        End If

        sr.Close()

    End Sub

    Sub DfileOutU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾃﾞｰﾀﾌｧｲﾙ名
        Dim aa$
        Dim i, ib, ie, ip, j As Integer

        Dim sw As New IO.StreamWriter(fxx$)
        aa$ = "プログラム名" + "," + "RotatForceVib"
        sw.WriteLine(aa$)
        sw.WriteLine(Title$)
        aa$ = "要素数" + "," + Str(Nelem)
        sw.WriteLine(aa$)
        aa$ = "剛性円板数" + "," + Str(Ndisc)
        sw.WriteLine(aa$)
        aa$ = "剛性円輪数" + "," + Str(Nwheel)
        sw.WriteLine(aa$)
        aa$ = "材料データ数" + "," + Str(Nmat)
        sw.WriteLine(aa$)
        aa$ = "拘束節点数" + "," + Str(Nbound)
        sw.WriteLine(aa$)
        aa$ = "バネ支持節点数" + "," + Str(Nspring)
        sw.WriteLine(aa$)
        aa$ = "減衰節点数" + "," + Str(Ndamp)
        sw.WriteLine(aa$)
        aa$ = "１節点当り変数の数" + "," + Str(Ndim)
        sw.WriteLine(aa$)
        aa$ = "材料データの成分数" + "," + Str(Mprop)
        sw.WriteLine(aa$)

        sw.WriteLine("要素データ (要素番号、材料番号、2節点番号、e、φe、ψ、φi)")
        For ie = 1 To Nelem
            aa$ = Str$(ie) + ",  " + Str$(NumMat(ie))
            For j = 1 To 2
                If j = 1 Then
                    ib = ie
                Else
                    ib = ie + 1
                End If
                aa$ = aa$ + ",  " + Str$(ib)
            Next j
            For j = 1 To 4
                aa$ = aa$ + "," + Str(DataElem(ie, j))
            Next j
            sw.WriteLine(aa$)
        Next ie

        aa$ = "節点データ（節点番号、X）"
        sw.WriteLine(aa$)
        For ip = 1 To Nelem + 1
            aa$ = Str$(ip) + ",  " + Str$(Xdata(ip))
            sw.WriteLine(aa$)
        Next ip

        If Ndisc > 0 Then
            aa$ = "剛性円板データ（円板番号、取付節点番号、密度、外径、内径、厚さ、e、φe、ψ、φi）"
            sw.WriteLine(aa$)
            For i = 1 To Ndisc
                aa$ = Str$(i)
                aa$ = aa$ + ",  " + Str$(KPdisc(i))
                For j = 1 To 8
                    aa$ = aa$ + ",  " + Str$(DiscData(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        If Nwheel > 0 Then
            aa$ = "剛性円輪データ"
            sw.WriteLine(aa$)
            For i = 1 To Nwheel
                aa$ = "円輪番号=" + "," + Str$(i)
                aa$ = aa$ + "," + "   取付節点番号=" + "," + Str$(KPwheel(i))
                sw.WriteLine(aa$)
                aa$ = "  密度、Lx、H、Hb1、Hb2、Hc1、Hc2"
                sw.WriteLine(aa$)
                aa$ = WheelData(i, 1)
                For j = 2 To 7
                    aa$ = aa$ + ",  " + Str$(WheelData(i, j))
                Next j
                sw.WriteLine(aa$)
                aa$ = "  D1、D2、Db1、Db2、Dc1、Dc2、e、φe、ψ、φi"
                sw.WriteLine(aa$)
                aa$ = WheelData(i, 8)
                For j = 9 To 17
                    aa$ = aa$ + ",  " + Str$(WheelData(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        aa$ = "材料データ (材料番号、ヤング率、ポアソン比、密度、直径、穴径）"
        sw.WriteLine(aa$)
        For i = 1 To Nmat
            aa$ = Str$(i)
            For j = 1 To Mprop
                aa$ = aa$ + ",  " + Str$(DataMat(i, j))
            Next j
            sw.WriteLine(aa$)
        Next i

        If Nbound > 0 Then
            aa$ = "拘束節点データ（拘束番号、節点番号、Y拘束、Z拘束）"
            sw.WriteLine(aa$)
            For i = 1 To Nbound
                aa = Str$(i) + ",  " + Str$(Nrest(i))
                For j = 1 To 2
                    aa$ = aa$ + ",  " + Str$(Jdirect(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        If Nspring > 0 Then
            aa$ = "バネ支持節点 (番号､節点番号､Ｋsy、Ｋsz）"
            sw.WriteLine(aa$)
            For ib = 1 To Nspring
                aa$ = Str$(ib) + ",  " + Str$(KPspring(ib))
                For j = 1 To 2
                    aa$ = aa$ + ",  " + Str$(Cspring(ib, j))
                Next j
                sw.WriteLine(aa$)
            Next ib
        End If

        If Ndamp > 0 Then
            aa$ = "減衰節点 (番号､節点番号､Ｃy、Ｃz）"
            sw.WriteLine(aa$)
            For ib = 1 To Ndamp
                aa$ = Str$(ib) + ",  " + Str$(KPdamp(ib))
                For j = 1 To 2
                    aa$ = aa$ + ",  " + Str$(Cdamp(ib, j))
                Next j
                sw.WriteLine(aa$)
            Next ib
        End If

        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Function DispMax(ByVal k1 As Integer, ByRef k2 As Integer, _
    ByRef tdb(,) As Single) As Single
        '変位の最大値
        'k1,k2･･･要素の開始、終了番号
        'dmax･･･X座標最小値
        'tdb(,)･･･節点の各座標方向変位
        Dim i, ie, j, k As Integer
        Dim b1, dmax As Single
        dmax = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    i = ie + j - 1
                    For k = 1 To 2
                        b1 = Math.Abs(tdb(i, k))
                        If b1 > dmax Then
                            dmax = b1
                        End If
                    Next k
                Next j
            End If
        Next ie
        Return dmax

    End Function

    Sub DispRateAnim(ByRef ratio As Single, ByVal rlen2 As Single)
        '変位作画倍率の設定
        'ratio･･･変位作画の倍率
        'rlen2･･･最大変位
        Dim rlen As Single

        Call AxisCheck(KE1, KE2, rlen, Xdata, Xdata, Xdata)
        '座標の最小・最大値のﾁｪｯｸ

        ratio = rlen2 / rlen
        Call AdjustRatio(ratio)     '変位比率の調整

    End Sub

    Sub DispRateSet(ByRef ratio As Single, ByVal tfdb(,) As Single)
        '変位作画倍率の設定
        'ratio･･･変位作画の倍率
        'tfdb(,)･･･節点の各座標変位
        Dim rlen, rlen2 As Single

        Call AxisCheck(KE1, KE2, rlen, Xdata, Xdata, Xdata)
        '座標の最小・最大値のﾁｪｯｸ

        rlen2 = DispMax(KE1, KE2, tfdb)
        ratio = rlen2 / rlen
        Call AdjustRatio(ratio)     '変位比率の調整

    End Sub

    Sub DrawFocusRect(ByRef gr As Graphics)
        'ﾌｫｰｶｽ矩形の描画
        Dim rect As Rectangle
        Dim p1, p2 As Point

        If Xmouse(1) <= Xmouse(2) Then
            p1.X = Xmouse(1)
            p2.X = Xmouse(2)
        Else
            p1.X = Xmouse(2)
            p2.X = Xmouse(1)
        End If
        If Ymouse(1) <= Ymouse(2) Then
            p1.Y = Ymouse(1)
            p2.Y = Ymouse(2)
        Else
            p1.Y = Ymouse(2)
            p2.Y = Ymouse(1)
        End If
        rect.X = p1.X
        rect.Y = p1.Y
        rect.Width = p2.X - p1.X
        rect.Height = p2.Y - p1.Y

        gr.DrawRectangle(Pens.DarkGray, rect)   '矩形の描画

    End Sub

    Sub DrawScale(ByVal dpixb As Single, ByVal dpiyb As Single, _
        ByVal xlen As Single, ByVal ylen As Single, ByVal xmin As Single, _
        ByVal xmax As Single, ByVal ymin As Single, ByVal ymax As Single)
        '作画倍率の計算
        'dpixb･･･X軸方向のdpi値
        'dpiyb･･･Y軸方向のdpi値
        'xlen･･･作画領域のX方向長さ
        'ylen･･･作画領域のY方向長さ
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim b1, b2, cx, cy As Single

        Cfix = dpiyb / dpixb

        b1 = xmax - xmin : b2 = ymax - ymin
        If Cfix < 1 Then
            If b1 <> 0 Then
                cx = xlen / b1
            End If
            If b2 <> 0 Then
                cy = ylen / b2 / Cfix
            End If
        Else
            If b1 <> 0 Then
                cx = Cfix * xlen / b1
            End If
            If b2 <> 0 Then
                cy = ylen / b2
            End If
        End If
        If b1 = 0 Then
            XYscale = cy
        ElseIf b2 = 0 Then
            XYscale = cx
        ElseIf cy < cx Then
            XYscale = cy
        Else
            XYscale = cx
        End If

    End Sub

    Sub Elem3DPrep()
        '構造図作画準備
        Dim ie As Integer
        ReDim KdrawElem(Nelem)
        ReDim Xmodel(Npoint), Ymodel(Npoint), Zmodel(Npoint)
        ReDim Xscreen(Npoint), Yscreen(Npoint)
        ReDim Xview(Npoint), Yview(Npoint), Zview(Npoint)

        KE1 = 1
        KE2 = Nelem
        KnumDisp = 0
        KnumSize = 8
        For ie = KE1 To KE2
            KdrawElem(ie) = 1
        Next ie

        Rlenmax = 0
        DistFactor = 0      '作画ﾊﾟﾗﾒｰﾀｰのｸﾘｱ

    End Sub

    Sub ElemSearch(ByRef k1 As Integer, ByRef k2 As Integer, _
    ByRef xsb() As Single, ByRef ysb() As Single)
        '拡大作画する要素の探索
        '両節点が指定領域にある要素を採用、作画要素なしの場合は元のﾃﾞｰﾀを保存
        'k1,k2･･･要素の開始、終了番号
        'xsb(),ysb()･･･節点のｽｸﾘｰﾝ座標
        Dim ib, ic, ie, j, kmax, kmin As Integer
        Dim b1, b2 As Single
        Dim kbuf(Nelem) As Integer

        ic = 0
        For ie = 1 To Nelem
            kbuf(ie) = KdrawElem(ie)
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    ib = ie + j - 1
                    b1 = xsb(ib) - Xmouse(1)
                    b2 = xsb(ib) - Xmouse(2)
                    If b1 >= 0 And b2 <= 0 Then
                        b1 = ysb(ib) - Ymouse(2)
                        b2 = ysb(ib) - Ymouse(1)
                        If b1 >= 0 And b2 <= 0 Then
                            If j = 2 Then
                                KdrawElem(ie) = 1
                                If ic = 0 Then
                                    ic = 1
                                    kmin = ie
                                    kmax = ie
                                ElseIf ie < kmin Then
                                    kmin = ie
                                ElseIf ie > kmax Then
                                    kmax = ie
                                End If
                            End If
                        Else
                            KdrawElem(ie) = 0
                            Exit For
                        End If
                    Else
                        KdrawElem(ie) = 0
                        Exit For
                    End If
                Next j
            End If
        Next ie
        k1 = kmin
        k2 = kmax
        If k1 = 0 Then
            For ie = 1 To Nelem
                KdrawElem(ie) = kbuf(ie)
            Next ie
        End If

    End Sub

    Private Sub ErrorDisp(ByVal kind As Integer, ByVal num As Integer,
        ByRef abuf() As String)
        'ｴﾗｰの表示
        'kind･･･打切り表示の有無（0:なし、1:有り）
        'num･･･ｴﾗｰの個数
        'abuf$()･･･ｴﾗｰの表示文
        Dim aa$, nc$
        Dim i As Integer
        nc$ = Chr(13) + Chr(10)
        aa$ = ""
        For i = 1 To num
            aa = aa$ + abuf(i) + nc$
        Next i
        If kind > 0 Then
            aa$ = aa$ + "データチェックを打切りました！！"
        End If
        MsgBox(aa$, 0, "データエラー")

    End Sub

    Sub FileName(ByVal fxx$, ByRef fxb$)
        'ﾌｧｲﾙ名fxx$の拡張子なしの部分fxb$を取り出す
        Dim i As Integer
        i = InStr(fxx$, ".")
        If i > 0 Then
            fxb$ = Left$(fxx$, i - 1)
        Else
            fxb$ = fxx$
        End If
    End Sub

    Sub ModelTrans(ByVal nb As Integer, ByRef xbuf() As Single, _
    ByRef ybuf() As Single, ByRef zbuf() As Single)
        'ﾓﾃﾞﾙ節点座標のｱﾌｨﾝ変換
        'nb･･･節点数
        'xbuf(),ybuf(),zbuf()･･･節点のX,Y,Z座標
        Dim ip, j, k As Integer
        Dim b1 As Single
        Dim buf(3) As Single
        For ip = 1 To nb
            buf(1) = xbuf(ip)
            buf(2) = ybuf(ip)
            buf(3) = zbuf(ip)
            For k = 1 To 3
                b1 = 0
                For j = 1 To 3
                    b1 = b1 + TransMat(k, j) * buf(j)
                Next j
                b1 = b1 + TransMat(k, 4)
                Select Case k
                    Case 1
                        xbuf(ip) = b1
                    Case 2
                        ybuf(ip) = b1
                    Case 3
                        zbuf(ip) = b1
                End Select
            Next k
        Next ip

    End Sub

    Sub MouseCalc()
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        Dim j As Integer
        Dim xa, xb, ya, yb As Single
        For j = 1 To 2
            xa = Xmouse(j)
            ya = Ymouse(j)
            If Cfix < 1 Then
                xb = (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
            Else
                xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale + YaxisMin
            End If
            Xmouse(j) = xb
            Ymouse(j) = yb
        Next j

    End Sub

    Sub MouseCalcS(ByVal xa As Single, ByRef xb As Single, ByVal ya As Single, ByRef yb As Single)
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        'xa,ya･･･ﾏｳｽ点座標
        'xb,yb･･･ﾓﾃﾞﾙ座標
        If Cfix < 1 Then
            xb = (xa - Xlower) / XYscale + XaxisMin
            yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
        Else
            xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
            yb = (Yupper - ya) / XYscale + YaxisMin
        End If

    End Sub

    Sub MousePointSearch(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef kp As Integer, ByVal xb As Single, ByVal yb As Single, _
    ByRef xmb() As Single, ByRef ymb() As Single)
        'ﾏｳｽ点の節点の探索
        'k1,k2･･･要素の開始、終了番号
        'kp･･･ﾏｳｽ点の節点番号
        'xb,yb･･･ﾏｳｽ点の作画座標位置
        'xmb(),ymb()･･･節点のｽｸﾘｰﾝ座標
        Dim ib, ie, j As Integer
        Dim b1, b2, b3, rlen2 As Single
        Dim kbuf(Npoint) As Integer

        kp = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    ib = ie + j - 1
                    If kbuf(ib) = 0 Then
                        kbuf(ib) = 1
                        b1 = xmb(ib) - xb
                        b2 = ymb(ib) - yb
                        b3 = b1 * b1 + b2 * b2
                        If kp = 0 Or b3 < rlen2 Then
                            kp = ib
                            rlen2 = b3
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub OptFileIn()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\RotFVMenu.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            KdataBackup = Val(sr.ReadLine)
            Kscreen = Val(sr.ReadLine)
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub OptFileOut()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\RotFVMenu.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(KdataBackup)
        sw.WriteLine(Kscreen)
        sw.Close()

    End Sub

    Sub PerspecCalc(ByVal k1 As Integer, ByVal k2 As Integer, ByVal rlenscreen As Single, _
    ByRef xvbuf() As Single, ByRef yvbuf() As Single, ByRef zvbuf() As Single, _
    ByRef xsbuf() As Single, ByRef ysbuf() As Single)
        'ｽｸﾘｰﾝへの透視座標の計算
        'k1,k2･･･要素の開始、終了番号
        'rlenscreen･･･透視座標原点からｽｸﾘｰﾝまでの距離
        'xvbuf(),yvbuf(),zvbuf()･･･視線座標のX,Y,Z
        'xsbuf(),ysbuf()･･･ｽｸﾘｰﾝ座標のX,Y
        Dim ib, ie, j As Integer
        Dim b1 As Single
        Dim kbuf(Npoint) As Integer

        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    ib = ie + j - 1
                    If kbuf(ib) = 0 Then
                        b1 = rlenscreen / zvbuf(ib)
                        xsbuf(ib) = b1 * xvbuf(ib)
                        ysbuf(ib) = b1 * (yvbuf(ib) - Eyeh)
                        kbuf(ib) = 1
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub RotateCalc(ByVal kind As Integer, ByRef theta As Single)
        '回転角の計算
        'kind･･･象限番号
        'theta･･･回転角（左ねじ正、°）
        Dim ix, iy, kb As Integer
        Dim b1 As Single

        ix = Xmouse(2) - Xmouse(1)
        iy = Ymouse(2) - Ymouse(1)
        Select Case kind
            Case 1
                If ix > 0 Then
                    If iy >= 0 Then
                        kb = -1
                    Else
                        kind = 0
                    End If
                Else
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = 1
                    End If
                End If
            Case 2
                If ix > 0 Then
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = -1
                    End If
                Else
                    If iy >= 0 Then
                        kb = 1
                    Else
                        kind = 0
                    End If
                End If
            Case 3
                If ix > 0 Then
                    If iy >= 0 Then
                        kb = 1
                    Else
                        kind = 0
                    End If
                Else
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = -1
                    End If
                End If
            Case 4
                If ix > 0 Then
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = 1
                    End If
                Else
                    If iy >= 0 Then
                        kb = -1
                    Else
                        kind = 0
                    End If
                End If
        End Select
        b1 = Math.Sqrt(ix * ix + iy * iy)
        theta = kb * 3.0 * b1 / 50.0

    End Sub

    Sub TmatMulti(ByRef tr1buf(,) As Single, ByRef tr2buf(,) As Single)
        '変換行列の掛算
        'tr1buf(,)･･･最初の変換行列
        'tr2buf(,)･･･次の変換行列および結果の変化行列
        Dim i, j, k As Integer
        Dim b1 As Single
        Dim buf(4, 4) As Single

        For i = 1 To 4
            For j = 1 To 4
                buf(i, j) = tr2buf(i, j)
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf(i, k) * tr1buf(k, j)
                Next k
                tr2buf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TimCal(ByRef ktim As Integer, ByRef timn$)
        'ﾀｲﾏｰ計算
        'ktim･･･時間継続指定（0:時間ｸﾘｱする、1:時間継続する）
        'timn$･･･時分秒に換算した経過時間
        Dim rtime As Date = Now
        Dim rhour, rmin, sec As Single
        If ktim > 0 Then
            sec = DateDiff(DateInterval.Second, Timst, rtime)
            rhour = Int(sec / 3600) : sec = sec - rhour * 3600
            rmin = Int(sec / 60) : sec = Int(sec - rmin * 60)
            timn$ = Str$(rhour) + "h" + Str$(rmin) + "min" + Str$(sec) + "s"
        Else
            Timst = rtime
        End If
    End Sub

    Sub TmatrixCalc(ByRef eyeh As Single, ByRef xrot As Single, _
        ByRef zrot As Single)
        'ﾓﾃﾞﾙの合成ｱﾌｨﾝ変換行列算出
        'eyeh･･･視点高さ（於ﾓﾃﾞﾙ座標）
        'xrot･･･ﾓﾃﾞﾙのX軸回りの回転角（右ねじ正、°）
        'zrot･･･ﾓﾃﾞﾙのz軸回りの回転角（右ねじ正、°）
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4) As Single
        ReDim TransMat(4, 4)

        b1 = Pai / 180
        thetax = xrot * b1
        thetaz = zrot * b1
        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf1(1, 1) = b1
        buf1(1, 2) = -b2
        buf1(2, 1) = b2
        buf1(2, 2) = b1
        buf1(3, 3) = 1
        buf1(4, 4) = 1
        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = -b2
        buf2(3, 2) = b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf2(i, k) * buf1(k, j)
                Next k
                TransMat(i, j) = b1
            Next j
        Next i
        For j = 1 To 4
            TransMat(3, j) = TransMat(3, j) - eyeh * TransMat(4, j)
        Next j
    End Sub

    Sub TmatrixCalD(ByVal xr As Single, ByVal yr As Single, ByVal zr As Single, _
    ByRef trbuf(,) As Single)
        'ﾓﾃﾞﾙの合成ｱﾌｨﾝ変換行列算出
        'Z,Y,X軸回りの回転
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        'trbuf(,)･･･変換ﾏﾄﾘｯｸｽ
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = -b2
        buf2(3, 2) = b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = b2
        buf3(2, 2) = 1
        buf3(3, 1) = -b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = -b2
        buf4(2, 1) = b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + trbuf(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalE(ByVal xr As Single, ByVal yr As Single, ByVal zr As Single, _
    ByRef trbuf(,) As Single)
        '座標軸の合成ｱﾌｨﾝ変換行列算出
        'Z,Y,X軸回りの回転
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        'trbuf(,)･･･変換ﾏﾄﾘｯｸｽ
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = b2
        buf2(3, 2) = -b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = -b2
        buf3(2, 2) = 1
        buf3(3, 1) = b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = b2
        buf4(2, 1) = -b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + trbuf(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmpFileIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\RotatForceVib.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            Fm$ = sr.ReadLine
            Fy$ = sr.ReadLine
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub


    Sub TmpFileOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\RotatForceVib.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(Fm$)
        sw.WriteLine(Fy$)
        sw.Close()

    End Sub

    Sub UFileAddA()
        'UFileAの追加処理
        Dim i, ic, j As Integer
        ic = 0
        For i = 2 To 6
            If UFileA(i) = UFileA(0) Then
                For j = i - 1 To 0 Step -1
                    UFileA(j + 1) = UFileA(j)
                Next j
                ic = 1
                Exit For
            End If
        Next i
        If ic = 0 Then
            For i = 5 To 0 Step -1
                UFileA(i + 1) = UFileA(i)
            Next i
        End If
        UseFileAOut()       'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し

    End Sub

    Sub UseFileAIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\RotVUFileA.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            i = 0
            Do Until sr.Peek = -1 'ﾌｧｲﾙ末尾まで
                i = i + 1
                UFileA(i) = sr.ReadLine
            Loop
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub UseFileAOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\RotVUFileA.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 6
            If UFileA(i) <> "" Then
                sw.WriteLine(UFileA(i))
            Else
                Exit For
            End If
        Next i
        sw.Close()

    End Sub

    Sub XYCalc(ByRef xb As Single, ByRef yb As Single)
        'X,Yの作画座標計算
        'xb･･･X座標値
        'yb･･･Y座標値
        Dim b1, b2 As Single
        b1 = xb - XaxisMin
        b2 = yb - YaxisMin
        If Cfix < 1 Then
            xb = b1 * XYscale + Xlower
            yb = Yupper - b2 * XYscale * Cfix
        Else
            xb = b1 * XYscale / Cfix + Xlower
            yb = Yupper - b2 * XYscale
        End If
    End Sub

End Module
